# script history: part of Merck distribution
# from 4.1.12 base
# modified by KG to fix CBCA2

namespace eval NvCBCA {
    proc LinkClust {{fr ""}} {
        global Clust int_flag NvCBCA ; # global vars
        set nClust $Clust(nClust) ; # the number of clusters
        set maxhits 12 
        set tol $NvCBCA(tolC13) ; # the C tol
        set int_flag 0 
        if {$fr != ""} { ; # if this argument is given create a cancel button
            button $fr.stopMatch -text Stop -command " 
            destroy $fr.stopMatch
            set int_flag 1
            "
            pack $fr.stopMatch -side left
            update
            grab $fr.stopMatch ; # restrict events in other windows
            
        }
        for {set i 0} {$i< $nClust} {incr i} { ; # initialize all the suc and pre clusters
            set Clust($i,suc) ""
            set Clust($i,pre) ""
        }
        for {set i 0} {$i< $nClust} {incr i} {
            puts -nonewline $i
            if {$int_flag} { ; # if Linking has been interrupted previously, do the following
                if {$fr != ""} {
                    destroy $fr.stopMatch
                }
                set int_flag 0
                return
            }
            if {[catch "MatchOnI $i All $tol" matches]} {
                destroy $fr.stopMatch
                error [list $matches]
            }
          #  MatchOnI $i All $tol
            puts ""
            set nhits [llength $matches]
            set NvCBCA(matchStatus) "$i $nhits"
            update
            if {$nhits > $maxhits} { set nhits $maxhits }
            set best [lrange $matches 0 [expr {$nhits-1}]]
            set Clust($i,suc) $best
        }
        destroy $fr.stopMatch
        GetPreClust
        
    }
    
    proc AddLinksToClust {iVal jVal tol maxhits} {
        global Clust int_flag
        set nClust $Clust(nClust)
        if {$iVal == "All"} {
            set iFirst 0
            set iLast [expr $nClust-1]
        } else {
            set iFirst $iVal
            set iLast $iVal
        }
        for {set i $iFirst} {$i<= $iLast} {incr i} {
            if {$int_flag} {
                set int_flag 0
                return
            }
            set matches [MatchOnI $i $jVal $tol]
            set nhits [llength $matches]
            if {$nhits > $maxhits} { set nhits $maxhits }
            set best [lrange $matches 0 [expr {$nhits-1}]]
            foreach hit $best {
                if {[CheckLinkExists $i suc $hit]<0} {
                    set hit [lreplace $hit 2 2 -1]
                    lappend Clust($i,suc) $hit
                    set Clust($i,suc) [lsort -index 1 -real -decreasing $Clust($i,suc)]
                    puts "$i Add suc $hit"
                    set j [lindex $hit 0]
                    set prehit [lreplace $hit 0 0 $i]
                    if {[CheckLinkExists $j pre $prehit]<0} {
                        set prehit [lreplace $prehit 2 2 -1]
                        lappend Clust($j,pre) $prehit
                        set Clust($j,pre) [lsort -index 1 -real -decreasing $Clust($j,pre)]
                        puts "$j Add pre $prehit "
                    }
                }
            }
        }
    }
    
    proc CheckLinkExists {iClust dir tlink} {
        global Clust
        set links $Clust($iClust,$dir)
        set peakst1 [lindex $tlink 4]
        set peakst2 [lindex $tlink 5]
        set i -1
        foreach link $links {
            incr i
            set peaks1 [lindex $link 4]
            set peaks2 [lindex $link 5]
            if {![string match $peaks1 $peakst1]} { continue }
            if {![string match $peaks2 $peakst2]} { continue }
            return $i
        }
        return -1
    }
    
    proc MatchOnI {i jVal tol} {
        global Clust NvCBCA
        set nClust $Clust(nClust)
        set matches ""
        if {$jVal == "All"} {
            set jFirst 0
            set jLast [expr $nClust-1]
        } else {
            set jFirst $jVal
            set jLast $jVal
        }
        for {set j $jFirst} {$j<= $jLast} {incr j} {
            update
            if {$i==$j} { continue }
            set results [eval $NvCBCA(matchScript) $i $j $tol [eval NvCBCAGetPkLists]] 
            foreach result $results {
                set score [lindex $result 0]
                set nmatch [lindex $result 1]
                set apeaks [lindex $result 2]
                set bpeaks [lindex $result 3]
                set restype [lindex $result 4]
                if {$nmatch > 0} {
                    lappend matches [list $j $score 0 $nmatch $apeaks $bpeaks $restype]
                }
            }
        }
        puts -nonewline " [llength $matches]"
        return [lsort -index 1 -real -decreasing $matches]
    }
    
    
    proc GetPreClust {} {
        global Clust
        set nClust $Clust(nClust)
        for {set i 0} {$i<$nClust} {incr i} {
            set Clust($i,pre) ""
        }
        for {set i 0} {$i<$nClust} {incr i} {
            foreach clust $Clust($i,suc) {
                set j [lindex $clust 0]
                set score [lindex $clust 1]
                set confirm [lindex $clust 2]
                set nmatch [lindex $clust 3]
                set apeaks [lindex $clust 4]
                set bpeaks [lindex $clust 5]
                set restype [lindex $clust 6]
                
                lappend Clust($j,pre) [list $i $score $confirm $nmatch $apeaks $bpeaks $restype]
            }
        }
        for {set i 0} {$i<$nClust} {incr i} {
            set Clust($i,pre) [lsort -index 1 -decreasing -real $Clust($i,pre)]
        }
    }

    proc MatchCBCA3 {i j tol iExp jExp kExp} {
        global Clust NvCBCA
        set HNtol 0.03
        set Ntol 0.3
        set aMin $tol
        set bMin $tol
        set aMatch 0
        set bMatch 0
        set iClust $Clust($i,$iExp)
        set jClust $Clust($j,$jExp)
        set aPeaks ""
        set bPeaks ""
        set aList ""
        set bList ""
        foreach iPeak $iClust {
            set intra [CheckIntra $i $iPeak $iExp $jExp $tol]
            if {$intra > 0} {
                if {$NvCBCA(noIntra)} {
                    continue
                }
                set iScore $NvCBCA(intraPenalty)
            } else {
                set iScore 0.0
            }
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $aMin} {
                    set aScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend aList "$aScore $iPPM $jPPM $iPeak $jPeak"
                    set aMatch 1
                }
            }
        }
        set iClust $Clust($i,$kExp)
        set jClust $Clust($j,$lExp)
        foreach iPeak $iClust {
            set intra [CheckIntra $i $iPeak $kExp $lExp $tol]
            if {$intra > 0} {
                if {$NvCBCA(noIntra)} {
                    continue
                }
                set iScore $NvCBCA(intraPenalty)
            } else {
                set iScore 0.0
            }
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $bMin} {
                    set bScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend bList "$bScore $iPPM $jPPM $iPeak $jPeak"
                    
                    set bMatch 1
                }
                
            }
        }
        if {!$aMatch && !$bMatch} { return "0.0 0" }
        set glys ""
        if {$aMatch} {
            set i 0
            foreach aPeaks $aList {
                if {[GlyPPM [lindex $aPeaks 1]]} {
                    set aScore [lindex $aPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    lappend result [list [expr {2.0*$aScore+1.0}] 1 $aPeak {} gly]
                    lappend glys $i
                }
                incr i
            }
        }
        foreach gly $glys {
            set aList [lreplace $aList $gly $gly ""]
        }
        if {$aMatch && $bMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                foreach bPeaks $bList {
                    if {$bPeaks == ""} {
                        continue
                    }
                    set HNscore 0.0
                    foreach ij "3 4" {
                        set aHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $aPeaks $ij]]
                        set bHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $bPeaks $ij]]
                        set aN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $aPeaks $ij]]
                        set bN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $bPeaks $ij]]
                        
                        set HNscore [expr {$HNscore +((1.0-(abs($aHN-$bHN) /$HNtol)) +(1.0-(abs($aN-$bN) /$Ntol))) /4.0}]
                    }
                    set aScore [lindex $aPeaks 0]
                    set bScore [lindex $bPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                    lappend result [list [expr {$aScore+$bScore+$HNscore}] 2 $aPeak $bPeak *]
                }
            }
            return $result
            
        }
        if {$aMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                set aScore [lindex $aPeaks 0]
                set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                lappend result [list $aScore 1 $aPeak {} *]
            }
            return $result
        }
        if {$bMatch} {
            foreach bPeaks $bList {
                set bScore [lindex $bPeaks 0]
                set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                lappend result [list $bScore 1 {} $bPeak *]
            }
            return $result
        }
    }
    
    proc MatchCBCA4 {i j tol iExp jExp kExp lExp} {
        global Clust NvCBCA
        set HNtol 0.03
        set Ntol 0.3
        set aMin $tol
        set bMin $tol
        set aMatch 0
        set bMatch 0
        set iClust $Clust($i,$iExp)
        set jClust $Clust($j,$jExp)
        set aPeaks ""
        set bPeaks ""
        set aList ""
        set bList ""
        foreach iPeak $iClust {
            set intra [CheckIntra $i $iPeak $iExp $jExp $tol]
            if {$intra > 0} {
                if {$NvCBCA(noIntra)} {
                    continue
                }
                set iScore $NvCBCA(intraPenalty)
            } else {
                set iScore 0.0
            }
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $aMin} {
                    set aScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend aList "$aScore $iPPM $jPPM $iPeak $jPeak"
                    set aMatch 1
                }
            }
        }
        set iClust $Clust($i,$kExp)
        set jClust $Clust($j,$lExp)
        foreach iPeak $iClust {
            set intra [CheckIntra $i $iPeak $kExp $lExp $tol]
            if {$intra > 0} {
                if {$NvCBCA(noIntra)} {
                    continue
                }
                set iScore $NvCBCA(intraPenalty)
            } else {
                set iScore 0.0
            }
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $bMin} {
                    set bScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend bList "$bScore $iPPM $jPPM $iPeak $jPeak"
                    
                    set bMatch 1
                }
                
            }
        }
        if {!$aMatch && !$bMatch} { return "0.0 0" }
        set glys ""
        if {$aMatch} {
            set i 0
            foreach aPeaks $aList {
                if {[GlyPPM [lindex $aPeaks 1]]} {
                    set aScore [lindex $aPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    lappend result [list [expr {2.0*$aScore+1.0}] 1 $aPeak {} gly]
                    lappend glys $i
                }
                incr i
            }
        }
        foreach gly $glys {
            set aList [lreplace $aList $gly $gly ""]
        }
        if {$aMatch && $bMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                foreach bPeaks $bList {
                    if {$bPeaks == ""} {
                        continue
                    }
                    set HNscore 0.0
                    foreach ij "3 4" {
                        set aHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $aPeaks $ij]]
                        set bHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $bPeaks $ij]]
                        set aN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $aPeaks $ij]]
                        set bN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $bPeaks $ij]]
                        
                        set HNscore [expr {$HNscore +((1.0-(abs($aHN-$bHN) /$HNtol)) +(1.0-(abs($aN-$bN) /$Ntol))) /4.0}]
                    }
                    set aScore [lindex $aPeaks 0]
                    set bScore [lindex $bPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                    lappend result [list [expr {$aScore+$bScore+$HNscore}] 2 $aPeak $bPeak *]
                }
            }
            return $result
            
        }
        if {$aMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                set aScore [lindex $aPeaks 0]
                set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                lappend result [list $aScore 1 $aPeak {} *]
            }
            return $result
        }
        if {$bMatch} {
            foreach bPeaks $bList {
                set bScore [lindex $bPeaks 0]
                set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                lappend result [list $bScore 1 {} $bPeak *]
            }
            return $result
        }
    }
    
    
    proc MatchCA {i j tol iExp jExp} {
        global Clust NvCBCA
        set HNtol 0.03
        set Ntol 0.3
        set aMin $tol
        set bMin $tol
        set aMatch 0
        set bMatch 0
        set iClust $Clust($i,$iExp)
        set jClust $Clust($j,$jExp)
        set aPeaks ""
        set bPeaks ""
        set aList ""
        set bList ""
        foreach iPeak $iClust {
            set intra [CheckIntra $i $iPeak $iExp $jExp $tol]
            if {$intra > 0} {
                if {$NvCBCA(noIntra)} {
                    continue
                }
                set iScore $NvCBCA(intraPenalty)
            } else {
                set iScore 0.0
            }
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $aMin} {
                    set aScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend aList "$aScore $iPPM $jPPM $iPeak $jPeak"
                    set aMatch 1
                }
            }
        }
        
        if {!$aMatch && !$bMatch} { return "0.0 0" }
        set glys ""
        
        if {$aMatch} {
            set i 0
            foreach aPeaks $aList {
                if {[GlyPPM [lindex $aPeaks 1]]} {
                    set aScore [lindex $aPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    lappend result [list [expr {2.0*$aScore}] 1 $aPeak {} gly]
                    lappend glys $i
                }
                incr i
            }
        }
        foreach gly $glys {
            set aList [lreplace $aList $gly $gly ""]
        }
        if {$aMatch && $bMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                foreach bPeaks $bList {
                    if {$bPeaks == ""} {
                        continue
                    }
                    set HNscore 0.0
                    foreach ij "3 4" {
                        set aHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $aPeaks $ij]]
                        set bHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $bPeaks $ij]]
                        set aN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $aPeaks $ij]]
                        set bN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $bPeaks $ij]]
                        
                        set HNscore [expr {$HNscore +((1.0-(abs($aHN-$bHN) /$HNtol)) +(1.0-(abs($aN-$bN) /$Ntol))) /4.0}]
                    }
                    set aScore [lindex $aPeaks 0]
                    set bScore [lindex $bPeaks 0]
                    set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                    set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                    lappend result [list [expr {$aScore+$bScore+$HNscore}] 2 $aPeak $bPeak *]
                }
            }
            return $result
            
        }
        if {$aMatch} {
            foreach aPeaks $aList {
                if {$aPeaks == ""} {
                    continue
                }
                set aScore [lindex $aPeaks 0]
                set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
                lappend result [list $aScore 1 $aPeak {} *]
            }
            return $result
        }
        if {$bMatch} {
            foreach bPeaks $bList {
                set bScore [lindex $bPeaks 0]
                set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
                lappend result [list $bScore 1 {} $bPeak *]
            }
            return $result
        }
    }
    
    
    
    proc MatchCBCA2 {i j tol iExp jExp} {
        global Clust NvCBCA
      #  puts "$i $j $tol $iExp $jExp" 
        set HNtol $NvCBCA(interList,tolHN)
        set Ntol $NvCBCA(interList,tolN)
        set aMin $tol
        set bMin $tol
        set aMatch 0
        set bMatch 0
        set result ""
        
        #        set iClust [lindex $Clust($i,$iExp) 0]
        #        set jClust [lindex $Clust($j,$jExp) 0]
        set alliClust $Clust($i,$iExp)
        set alljClust $Clust($j,$jExp)
        
        foreach iPeak $alliClust {
            foreach jPeak $alljClust {
                set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set iint [eval nv_peak elem int $iPeak]
                set jint [eval nv_peak elem int $jPeak]
                set delta [expr abs($iPPM-$jPPM)]
                
                if {($iint>0.0) && ($jint>0)} {
                    if {$delta < $aMin} {
                        set aMin $delta
                        set aScore [expr 1.0-($delta/$tol)]
                        set aPPM $iPPM
                        set aPeaks "$iPeak $jPeak"
                        set aMatch 1
                    }
                } else {
                    if {$delta < $bMin} {
                        set bMin $delta
                        set bScore [expr 1.0-($delta/$tol)]
                        set bPeaks "$iPeak $jPeak"
                        set bMatch 1
                    }
                }
            }
        }
        if {!$aMatch && !$bMatch} { return "0 0.0" }
        if {$aMatch && $bMatch} {
            set HNscore 0.0
            foreach ij "0 1" {
                set aHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $aPeaks $ij]]
                set bHN [eval nv_peak elem $NvCBCA(HNdim).P [lindex $bPeaks $ij]]
                set aN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $aPeaks $ij]]
                set bN [eval nv_peak elem $NvCBCA(Ndim).P [lindex $bPeaks $ij]]
                set HNscore [expr $HNscore + ((1.0-(abs($aHN-$bHN)/$HNtol))+(1.0-(abs($aN-$bN)/$Ntol)))/4.0]
            }
            lappend result [list [expr $aScore+$bScore+$HNscore] 2 $aPeaks $bPeaks *]
            return $result
        }
        if {$aMatch} {
            if {[GlyPPM $aPPM]} {
                lappend result [list [expr 2.0*$aScore] 1 $aPeaks {} gly]
                return $result
            } else {
                lappend result [list $aScore 1 $aPeaks {} *]
                return $result
            }
        }
        if {$bMatch} {
        lappend result [list $bScore 1 {} $bPeaks *] }
        return $result
    }
    
    
    proc MatchNOESelf {tol i j iExp} {
        global Clust NvCBCA
        set HNtol 0.03
        set Ntol 0.3
        set aMin $tol
        set aMatch 0
        set iClust $Clust($i,$iExp)
        set jClust $Clust($j,$iExp)
        set aPeaks ""
        set aList ""
        foreach iPeak $iClust {
            set iScore 0.0
            set iPPM [eval nv_peak elem $NvCBCA(Cdim).P $iPeak]
            set iIntensity [eval nv_peak elem int $iPeak]
            foreach jPeak $jClust {
                set jPPM [eval nv_peak elem $NvCBCA(Cdim).P $jPeak]
                set jIntensity [eval nv_peak elem int $jPeak]
                set delta [expr {abs($iPPM-$jPPM)}]
                if {$delta < $aMin} {
                    set aScore [expr {1.0-($delta/$tol) -$iScore}]
                    lappend aList "$aScore $iPPM $jPPM $iPeak $jPeak"
                    set aMatch 1
                }
            }
        }
        
        if {$aMatch} {
            set scoreSum 0.0
            set nPeaks 0
            set aList [lsort -index 0 -real $aList]
            set aPeaks [lindex $aList 0]
            set aPeak "[lindex $aPeaks 3] [lindex $aPeaks 4]"
            if {[llength $aList] > 1} {
                set bPeaks [lindex $aList 1]
                set bPeak "[lindex $bPeaks 3] [lindex $bPeaks 4]"
            } else {
                set bPeak {}
            }
            foreach aPeaks $aList {
                set aScore [lindex $aPeaks 0]
                set scoreSum [expr {$scoreSum+$aScore}]
                incr nPeaks
            }
            set result [list $scoreSum $nPeaks $aPeak $bPeak *]
            return $result
        }
    }
    
    proc GlyPPM {ppm} {
        if {$ppm==""} { return 0 }
        if {($ppm>39.0) &&($ppm<49.0)} { return 1 } else { return 0 }
    }
    
    proc CheckIntra {iClust intraPeak intraExp interExp tol} {
        global NvCBCA Clust
        set nIntra [llength $Clust($iClust,$intraExp)]
        set nInter [llength $Clust($iClust,$interExp)]
        if {($nIntra != 2) ||($nInter !=1)} {
            return 0
        }
        set index [lsearch $Clust($iClust,$intraExp) $intraPeak]
        if {$index == 0} {
            set other 1
        } else {
            set other 0
        }
        set intraPeak2 [lindex $Clust($iClust,$intraExp) $other]
        set interPeak $Clust($iClust,$interExp)
        set intraPPM [eval nv_peak elem $NvCBCA(Cdim).P $intraPeak]
        set intraPPM2 [eval nv_peak elem $NvCBCA(Cdim).P $intraPeak2]
        set interPPM [eval nv_peak elem $NvCBCA(Cdim).P $interPeak]
        set delta [expr {abs($intraPPM-$interPPM)}]
        set delta2 [expr {abs($intraPPM2-$interPPM)}]
        if {($delta < $tol) &&($delta<$delta2)} {
            return 1
        } else {
            return -1
        }
        
    }
    proc FilterPeaks {} {
        global Clust NvCBCA
        nv_peak template $NvCBCA(filter,list) $NvCBCA(HNdim) $NvCBCA(filter,tolHN) $NvCBCA(Ndim) $NvCBCA(filter,tolN)
        foreach peaklist $NvCBCA(filterLists) {
            set npeaks 0
            set sweep_ppm [expr [lindex [eval nv_peak sw $peaklist] 2] / \
            [lindex [eval nv_peak sf $peaklist] 2]]
	   
            foreachpeak ipeak $peaklist {
                set hn [nv_peak elem $NvCBCA(HNdim).P $peaklist.$ipeak]
                set n [nv_peak elem $NvCBCA(Ndim).P $peaklist.$ipeak]
                set peaks [nv_peak find $NvCBCA(filter,list) $hn $n]
	#	puts "$hn $n"
                lappend peaks [nv_peak find $NvCBCA(filter,list) $hn [expr $n+$sweep_ppm]] ;# allow for folding
                lappend peaks [nv_peak find $NvCBCA(filter,list) $hn [expr $n-$sweep_ppm]]
		#puts $peaks
                if {$peaks=="{} {}"} {

                    puts "$peaklist.$ipeak marked for deletion"
                    nv_peak elem stat $peaklist.$ipeak -1
                    incr npeaks
                }
            }
            puts " -> Marked $npeaks peaks from list $peaklist"
        }
    }
    
    proc UnConfirmAll {} {
        global Clust NvCBCA
        set nClust $Clust(nClust)
        for {set iClust 0} {$iClust< $nClust} {incr iClust} {
            foreach dir "suc pre" {
                set newClusts ""
                foreach clust $Clust($iClust,$dir) {
                    set newClust [lreplace $clust 2 2 0]
                    lappend newClusts $newClust
                }
                set Clust($iClust,$dir) $newClusts
            }
        }
    }
    
    
    proc AutoConfirm {limit} {
        global Clust NvCBCA
        set nClust $Clust(nClust)
        foreach dir "suc pre" rdir "pre suc" {
            set nConfirm 0
            
            for {set iClust 0} {$iClust< $nClust} {incr iClust} {
                set nIGood 0
                set nJGood 0
                set i 0
                set skip 0
                foreach clust $Clust($iClust,$dir) {
                    set status [lindex $clust 2]
                    if {$status > 0} {
                        set skip 1
                        break
                    }
                }
                if {$skip} { continue }
                set okI 0
                set okJ 0
                foreach clust $Clust($iClust,$dir) {
                    set score [lindex $clust 1]
                    set pclust [lindex $clust 0]
                    set sclusters $Clust($pclust,$rdir)
                    if {[llength $sclusters]>0} {
                        set sclust [lindex [lindex $sclusters 0] 0]
                    } else {
                        set sclust -1
                    }
                    set score [lindex $clust 1]
                    if {$score < $limit} {
                        break
                    }
                    if {$sclust == $iClust} {
                        set okI 1
                        incr nIGood
                        set iElem $i
                    }
                    incr i
                }
                if {!$okI} { continue }
                set jClust [lindex [lindex $Clust($iClust,$dir) $iElem] 0]
                if {$jClust == ""} { continue }
                
                foreach clust $Clust($jClust,$rdir) {
                    set score [lindex $clust 1]
                    set pclust [lindex $clust 0]
                    set sclusters $Clust($pclust,$dir)
                    if {[llength $sclusters]>0} {
                        set sclust [lindex [lindex $sclusters 0] 0]
                    } else {
                        set sclust -1
                    }
                    set score [lindex $clust 1]
                    if {$score < $limit} {
                        break
                    }
                    if {$sclust == $jClust} {
                        set okJ 1
                        incr nJGood
                    }
                }
                if {!$okJ} { continue }
                
                foreach ocluster $Clust($jClust,$rdir) {
                    set statVar [lindex $ocluster 2]
                    if {$statVar > 0} {
                        set nIGood 0
                        break
                    }
                }
                
                if {($nJGood == 1) && ($nIGood == 1)} {
                    incr nConfirm
                    NvCBCADoConfirm $iClust $iElem confirm $dir
                }
            }
            puts $nConfirm
        }
        
    }
    
    proc AutoConfirm2 {limit {limit1 0.95}} {
        global Clust NvCBCA
        set nClust $Clust(nClust)
        foreach dir "suc pre" rdir "pre suc" {
            set nConfirm 0
            
            for {set iClust 0} {$iClust< $nClust} {incr iClust} {
                set nIGood 0
                set iElem -1
                set skip 0
                set okI 0
                set okJ 0
                foreach clust $Clust($iClust,$dir) {
                    incr iElem
                    if {[lindex $clust 2] > 0} {
                        incr nIGood
                        continue
                    }
                    set score [lindex $clust 1]
                    set resType [lindex $clust end]
                    set nMatch [lindex $clust 3]
                    if {$score < $limit1} {
                        break
                    }
                    if {(($nMatch == 2) ||($resType == "gly")) &&($score < $limit)} {
                        continue
                    }
                    if {($nMatch == 1) && ($nIGood > 0)} {
                        break
                    }
                    NvCBCADoConfirm $iClust $iElem confirm $dir
                    incr nConfirm
                    incr nIGood
                }
            }
            puts $nConfirm
        }
        
    }
    
    
    proc ClassifyClusterLinks {{clustFirst 0} {clustLast 10000}} {
        global Clust NvCBCA
        
        set peakStart 4
        set nClust $Clust(nClust)
        if {$clustLast >= $nClust} {
            set clustLast [expr $nClust -1]
        }
        foreach dir "suc pre" rdir "pre suc" whichPeak "0 1" {
            set nCombs 1
            set nMult 0
            
            for {set iClust $clustFirst} {$iClust <= $clustLast} {incr iClust} {
                
                set clust0 [lindex $Clust($iClust,$dir) 0]
                set peakLast [expr {[llength $clust0]-2}]
                set refPair(0,peaks) ""
                set refPair(0,n) 0
                foreach peakPair [lrange $clust0 $peakStart $peakLast] {
                    lappend refPair(0,peaks) $peakPair
                }
                
                set pairList 0
                
                set n [llength $Clust($iClust,$dir)]
                set clustMatches ""
                set i 0
                set k 0
                for {set k 0} {$k<$n} {incr k} {
                    set i $k
                    set indices ""
                    set ok 0
                    set belowScore 0
                    set peaksUsed ""
                    for {set j $k} {$j<$n} {incr j} {
                        set clust [lindex $Clust($iClust,$dir) $j]
                        set confirmed [lindex $clust 2]
                        if {$confirmed <= 0} {
                            continue
                        }
                        set used 0
                        foreach peakPair [lrange $clust $peakStart $peakLast] {
                            foreach peak $peakPair {
                                if {[lsearch $peaksUsed $peak] != -1} {
                                    set used 1
                                    break
                                }
                            }
                            if {$used} { break }
                        }
                        if {!$used} {
                            foreach peakPair [lrange $clust $peakStart $peakLast] {
                                foreach peak $peakPair {
                                    lappend peaksUsed $peak
                                }
                            }
                            lappend indices $j
                            set ok 1
                        }
                    }
                    if {$ok} {
                        lappend clustMatches $indices
                    } else {
                        break
                    }
                }
                if {[llength $clustMatches] > 0} {
                    set nCombs [expr $nCombs*[llength $clustMatches]]
                    puts "$iClust $clustMatches $nCombs"
                    if {[llength $clustMatches] > 1} {
                        incr nMult
                    }
                }
                
            }
            puts "$nCombs $nMult"
        }
        
    }
    
    proc TreeLinksOld {{clustFirst 0} {clustLast 10000}} {
        global Clust NvCBCA ClustPath PeaksUsed Fragment Fragment2 Fragsuc Fragpre FragUniq
        
        set peakStart 4
        set nClust $Clust(nClust)
        if {$clustLast >= $nClust} {
            set clustLast [expr $nClust -1]
        }
        set nFrag2 0
        catch "unset Fragment2"
        catch "unset FragUniq"
        for {set iClust $clustFirst} {$iClust <= $clustLast} {incr iClust} {
            puts $iClust
            foreach dir "suc pre" rdir "pre suc" whichPeak "0 1" {
                catch "unset Fragment"
                set Fragment(nFrag) 0
                set Fragment(AtmTypes) "CA CB"
                catch "unset ClustPath"
                catch "unset PeaksUsed"
                set ClustPath(nPaths) 0
                set ClustPath(current) ""
                set ClustPath(current,elems) ""
                set ClustPath(added) 0
                
                VisitClust $iClust 0 $dir
                set Frags($dir,nFrag) $Fragment(nFrag)
                foreach elem [array names Fragment] {
                    set Frag${dir}($elem) $Fragment($elem)
                }
            }
            parray Fragsuc
            parray Fragpre
            for {set i 0} {$i < $Fragsuc(nFrag)} {incr i} {
                for {set j 0} {$j < $Fragpre(nFrag)} {incr j} {
                    set end2 [expr [llength $Fragpre($j,clst)]-1]
                    set srch [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst) [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                    if {[info exists FragUniq($srch)]} { continue }
                    
                    foreach atmType $Fragment(AtmTypes) {
                        set Fragment(-1,$atmType) [concat [lrange $Fragpre($j,$atmType) 0 $end2] $Fragsuc($i,$atmType)]
                        set Fragment(-1,$atmType,int) [concat [lrange $Fragpre($j,$atmType,int) 0 $end2] $Fragsuc($i,$atmType,int)]
                        set Fragment(-1,$atmType,peaks) [concat [lrange $Fragpre($j,$atmType,peaks) 0 $end2] $Fragsuc($i,$atmType,peaks)]
                    }
                    
                    set Fragment(-1,clst) [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst)]
                    set Fragment(-1,link) [concat [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                    set Fragment(-1,resType) [concat [lrange $Fragpre($j,resType) 0 $end2] $Fragsuc($i,resType)]
                    GetShifts -1
                    set result [ScoreFrag -1]
                    if {[string trim $result] == ""} {
                        puts "no result"
                        continue
                    }
                    set minRatio 0.095
                    set FragSize [llength $Fragment(-1,clst)]
                    set gWorst [lindex $result 6]
                    set Ratio [lindex $result 5]
                    set firstRes [lindex $result 1]
                    set TScore [lindex $result 2]
                    set MinScore [expr {$TScore/$FragSize}]
                    set first2Res [lindex $result 3]
                    set Min2Score [expr [lindex $result 4]/$FragSize]
                    if {($TScore < 100.0)} {
                        set FragUniq($srch) 1
                        foreach atmType $Fragment(AtmTypes) {
                            set Fragment2($nFrag2,$atmType) [concat [lrange $Fragpre($j,$atmType) 0 $end2] $Fragsuc($i,$atmType)]
                            set Fragment2($nFrag2,$atmType,int) [concat [lrange $Fragpre($j,$atmType,int) 0 $end2] $Fragsuc($i,$atmType,int)]
                            set Fragment2($nFrag2,$atmType,peaks) [concat [lrange $Fragpre($j,$atmType,peaks) 0 $end2] $Fragsuc($i,$atmType,peaks)]
                        }
                        set Fragment2($nFrag2,clst) [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst)]
                        set Fragment2($nFrag2,link) [concat [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                        set Fragment2($nFrag2,resType) [concat [lrange $Fragpre($j,resType) 0 $end2] $Fragsuc($i,resType)]
                        incr nFrag2
                    }
                    
                }
            }
        }
        set Fragment2(nFrag) $nFrag2
        set Fragment2(AtmTypes) "CA CB"
        catch "unset Fragment"
        set Fragment(nFrag) 0
        set Fragment(AtmTypes) "CA CB"
        set nFrag 0
        for {set j 0} {$j<$nFrag2} {incr j} {
            foreach atmType $Fragment2(AtmTypes) {
                set Fragment($nFrag,$atmType) $Fragment2($j,$atmType)
                set Fragment($nFrag,$atmType,int) $Fragment2($j,$atmType,int)
                set Fragment($nFrag,$atmType,peaks) $Fragment2($j,$atmType,peaks)
            }
            set Fragment($nFrag,clst) $Fragment2($j,clst)
            set Fragment($nFrag,link) $Fragment2($j,link)
            set Fragment($nFrag,resType) $Fragment2($j,resType)
            incr nFrag
        }
        set Fragment(nFrag) $nFrag
    }
    
    proc TreeLinks {{clustFirst 0} {clustLast 10000}} {
        global Clust NvCBCA ClustPath PeaksUsed Fragment Fragment2 Fragsuc Fragpre FragUniq ClustUsed
        
        set peakStart 4
        set nClust $Clust(nClust)
        if {$clustLast >= $nClust} {
            set clustLast [expr $nClust -1]
        }
        set nFrag2 0
        catch "unset Fragment2"
        catch "unset FragUniq"
        catch "unset ClustUsed"
        for {set iClust $clustFirst} {$iClust <= $clustLast} { incr iClust} {
           # puts "$iClust **************************"
            catch "unset Fragpre"
            catch "unset Fragsuc"
            foreach dir "suc pre" rdir "pre suc" whichPeak "0 1" {
                catch "unset Fragment"    ; #reinitialize all vars for each pass
                set Fragment(nFrag) 0
                set Fragment(AtmTypes) "CA CB"
                catch "unset ClustPath"
                catch "unset PeaksUsed"
                set ClustPath(nPaths) 0
                set ClustPath(current) ""
                set ClustPath(current,elems) ""
                set ClustPath(current,scores) ""
                set ClustPath(current,seqs) ""
                set ClustPath(added) 0
                
	#	puts "VisitClust" 
                VisitClust $iClust 0 $dir
                set Frags($dir,nFrag) $Fragment(nFrag)
                foreach elem [array names Fragment] {
                    set Frag${dir}($elem) $Fragment($elem)
                }
            }
            for {set i 0} {$i < $Fragsuc(nFrag)} {incr i} {
                set lastResidues [lindex $Fragsuc($i,seqs) end]
                set lengthSuc [llength $Fragsuc($i,clst)]
                for {set j 0} {$j < $Fragpre(nFrag)} {incr j} {
                    set firstResidues [lindex $Fragpre($j,seqs) 0]
                    set lengthPre [llength $Fragpre($j,clst)]
                    set okMatch 0
                    set seqs ""
                    foreach lastRes $lastResidues {
                        foreach firstRes $firstResidues {
                            if {[expr {($firstRes+$lengthPre)==($lastRes-$lengthSuc+1)}]} {
                                set okMatch 1
                                lappend seqs [expr {$firstRes}]
                            }
                        }
                    }
                    if {!$okMatch} {continue}
                    set end2 [expr [llength $Fragpre($j,clst)]-1]
                    set srch [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst) [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                    if {[info exists FragUniq($srch)]} {continue}
                    
                    set Fragment(-1,clst) [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst)]
                    set Fragment(-1,link) [concat [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                    
                    set result [ScoreFrag3 -1 $seqs]
                    set minRatio 0.095
                    set FragSize [llength $Fragment(-1,clst)]
                    
                    if {[llength $result] > 0} {
                        set TScore [lindex [lindex $result 0] 1]
                        set MinScore [expr {$TScore/$FragSize}]
                    } else {
                        set TScore 1000.0
                    }
                    if {($TScore < 100.0) } {
                        set FragUniq($srch) 1
                        set Fragment2($nFrag2,seqScores) $result
                        set Fragment2($nFrag2,clst) [concat [lrange $Fragpre($j,clst) 0 $end2] $Fragsuc($i,clst)]
                        set Fragment2($nFrag2,link) [concat [lrange $Fragpre($j,link) 0 $end2] $Fragsuc($i,link)]
                        #set Fragment2($nFrag2,resType) [concat [lrange $Fragpre($j,resType) 0 $end2] $Fragsuc($i,resType)]
                        incr nFrag2
                    }
                    
                }
            }
        }
        set Fragment2(nFrag) $nFrag2
        set Fragment2(AtmTypes) "CA CB"
        catch "unset Fragment"
        set Fragment(nFrag) 0
        set Fragment(AtmTypes) "CA CB"
        set nFrag 0
        for {set j 0} {$j<$nFrag2} {incr j} {
            if {0} {
                foreach atmType $Fragment2(AtmTypes) {
                    set Fragment($nFrag,$atmType) $Fragment2($j,$atmType)
                    set Fragment($nFrag,$atmType,int) $Fragment2($j,$atmType,int)
                    set Fragment($nFrag,$atmType,peaks) $Fragment2($j,$atmType,peaks)
                }
            }
            
            set Fragment($nFrag,clst) $Fragment2($j,clst)
            set Fragment($nFrag,link) $Fragment2($j,link)
            #set Fragment($nFrag,resType) $Fragment2($j,resType)
            set Fragment($nFrag,seqScores) $Fragment2($j,seqScores)
            incr nFrag
        }
        set Fragment(nFrag) $nFrag
    }
    
    proc VisitClustOld {iClust iElem dir} {
        global Clust ClustPath PeaksUsed Fragment
        set peakStart 4
        if {$dir == "suc"} {
            set rdir pre
        } else {
            set rdir suc
        }
        set iElem -1
        foreach clust "$Clust($iClust,$dir) {}" {
            if {$clust == {}} {
                set ClustPath(added) 1
                break
            }
            incr iElem
            set confirmed [lindex $clust 2]
            set nmatch [lindex $clust 3]
            set resType [lindex $clust end]
            if {($confirmed <= 0) ||(($nmatch == 1) &&($resType != "gly"))} {
                continue
            } else {
                set used 0
                set peakLast [expr {[llength $clust]-2}]
                foreach peakPair [lrange $clust $peakStart $peakLast] {
                    if {$peakPair == {}} { continue }
                    if {$dir == "suc"} {
                        set peak [lindex $peakPair 1]
                    } else {
                        set peak [lindex $peakPair 0]
                    }
                    
                    if {[info exists PeaksUsed($peak)] && $PeaksUsed($peak)} {
                        set used 1
                        break
                    }
                    if {$used} { break }
                }
                if {$used} { continue }
                foreach peakPair [lrange $clust $peakStart $peakLast] {
                    if {$dir == "suc"} {
                        set peak [lindex $peakPair 1]
                    } else {
                        set peak [lindex $peakPair 0]
                    }
                    set PeaksUsed($peak) 1
                }
                set jClust [lindex $clust 0]
                set newClust $Clust($jClust,$rdir)
                set jElem 0
                foreach nclust $newClust {
                    set ok 1
                    foreach peakPair1 [lrange $clust $peakStart $peakLast] peakPair2 [lrange $nclust $peakStart $peakLast] {
                        if {$peakPair1 != $peakPair2} {
                            set ok 0
                            break
                        }
                    }
                    if {$ok} {
                        break
                    }
                    incr jElem
                }
                
                
                set ClustPath(added) 1
                if {$dir == "suc"} {
                    lappend ClustPath(current) "$iClust $jClust"
                    lappend ClustPath(current,elems) "$iElem $jElem"
                } else {
                    set ClustPath(current) [linsert $ClustPath(current) 0 "$jClust $iClust"]
                    set ClustPath(current,elems) [linsert $ClustPath(current,elems) 0 "$jElem $iElem"]
                }
                VisitClust $jClust $jElem $dir
            }
        }
        if {[llength $ClustPath(current)] == 0} { return }
        if {$ClustPath(added)} {
            set minRatio 0.095
            set nFrag $Fragment(nFrag)
            set Fragment(-1,clst) $ClustPath(current)
            set Fragment(-1,link) $ClustPath(current,elems)
            if {0} {
                if {$dir == "suc"} {
                    set firstClust [lindex [lindex $Fragment(-1,clst) 0] 0]
                    if {$firstClust != {}} {
                        set firstLink {}
                        set Fragment(-1,clst) [linsert $Fragment(-1,clst) 0 "{} $firstClust"]
                        set Fragment(-1,link) [linsert $Fragment(-1,link) 0 "{} $firstLink"]
                    }
                }\
                elseif {$dir == "pre"} {
                    set lastClust [lindex [lindex $Fragment(-1,clst) end] 1]
                    if {$lastClust != {}} {
                        set lastLink {}
                        lappend Fragment(-1,clst) "$lastClust {}"
                        lappend Fragment(-1,link) "$lastLink {}"
                    }
                }
            }
            set FragSize [llength $ClustPath(current)]
            GetShifts -1
            parray Fragment -1,*
            set result [ScoreFrag -1]
            parray Fragment -1,*
            if {[string trim $result] == ""} {
                puts "no result"
                return
            }
            
            set gWorst [lindex $result 6]
            set Ratio [lindex $result 5]
            set firstRes [lindex $result 1]
            set TScore [lindex $result 2]
            set MinScore [expr {$TScore/$FragSize}]
            set first2Res [lindex $result 3]
            set Min2Score [expr [lindex $result 4]/$FragSize]
            if {($TScore < 100.0)} {
                set Fragment($nFrag,clst) $ClustPath(current)
                set Fragment($nFrag,link) $ClustPath(current,elems)
                if {$dir == "pre"} {
                    set firstClust [lindex [lindex $Fragment($nFrag,clst) 0] 0]
                    if {$firstClust != {}} {
                        set firstLink {}
                        set Fragment($nFrag,clst) [linsert $Fragment($nFrag,clst) 0 "{} $firstClust"]
                        set Fragment($nFrag,link) [linsert $Fragment($nFrag,link) 0 "{} $firstLink"]
                    }
                }\
                elseif {$dir == "pre"} {
                    set lastClust [lindex [lindex $Fragment($nFrag,clst) end] 1]
                    if {$lastClust != {}} {
                        set lastLink {}
                        lappend Fragment($nFrag,clst) "$lastClust {}"
                        lappend Fragment($nFrag,link) "$lastLink {}"
                    }
                }
                GetShifts $nFrag
                incr Fragment(nFrag)
            }
            
            
        }
        set kClust [lindex [lindex $ClustPath(current) end] 0]
        foreach clust $Clust($kClust,$dir) {
            set peakLast [expr {[llength $clust]-2}]
            set confirmed [lindex $clust 2]
            if {$confirmed <= 0} { continue }
            foreach peakPair [lrange $clust $peakStart $peakLast] {
                if {$dir == "suc"} {
                    set peak [lindex $peakPair 1]
                } else {
                    set peak [lindex $peakPair 0]
                }
                set PeaksUsed($peak) 0
            }
        }
        
        if {$dir == "suc"} {
            set ClustPath(current) [lrange $ClustPath(current) 0 [expr {[llength $ClustPath(current)]-2}]]
            set ClustPath(current,elems) [lrange $ClustPath(current,elems) 0 [expr {[llength $ClustPath(current,elems)]-2}]]
        } else {
            set ClustPath(current) [lrange $ClustPath(current) 1 end]
            set ClustPath(current,elems) [lrange $ClustPath(current,elems) 1 end]
        }
        
        set ClustPath(added) 0
    }
    
    
    proc VisitClust {iClust iElem dir} {
        global Clust ClustPath PeaksUsed Fragment ClustUsed pastResult
        set peakStart 4
        if {[info exists ClustUsed($iClust.$iElem.$dir)]} {
            #   return
        }
        set ClustUsed($iClust.$iElem.$dir) 1
        if {$dir == "suc"} {
            set rdir pre
        } else {
            set rdir suc
        }
        set iElem -1
        set nAdded 0
        set gotEnd 0
        set pastResult 1
        foreach clust "$Clust($iClust,$dir) {}" {
            if {$clust == {}} {
              #  puts stop
                break
            }
            incr iElem
            set confirmed [lindex $clust 2]
            set nmatch [lindex $clust 3]
            set resType [lindex $clust end]
            if {($confirmed <= 0) || (($nmatch == 0) && ($resType != "gly"))} {
                continue
            } else {
                set used [CheckPeakUse $clust $peakStart $dir] ; #check if the matching peaks in the pre or suc clust are marked used
                if {$used} {continue}
                set peakLast [expr {[llength $clust]-2}]
                
                set jClust [lindex $clust 0]
                set newClust $Clust($jClust,$rdir)
                set jElem 0
                foreach nclust $newClust {
                    set ok 1
                    foreach peakPair1 [lrange $clust $peakStart $peakLast] peakPair2 [lrange $nclust $peakStart $peakLast] {
                        if {$peakPair1 != $peakPair2} { ; # make sure the peak pairs are the same
                            set ok 0
                            break
                        }
                    }
                    if {$ok} {
                        break
                    }
                    incr jElem ; #if they aren't move on to the next Elem
                }
                set nextSeqs ""
                set nextScores ""
                set addClust 0
                if {$dir == "suc"} {
                    set seqs [lindex $ClustPath(current,seqs) end]
                    set scores [lindex $ClustPath(current,scores) end]
                    if {$seqs == {}} {
                        set nextSeqs $Clust($iClust,$iElem,$dir,seqs)
                        set nextScores $Clust($iClust,$iElem,$dir,scores)
                    } else {
                        foreach seq $seqs score $scores {
                            set nextSeq [expr $seq+1]
                            set index [lsearch $Clust($iClust,$iElem,$dir,seqs) $nextSeq]
                            if {$index != -1} {
                                lappend nextSeqs $nextSeq
                                lappend nextScores [lindex $Clust($iClust,$iElem,$dir,scores) $index]
                            }
                        }
                    }
                    
                    if {[llength $nextSeqs] > 0} {
                        lappend ClustPath(current) "$iClust $jClust"
                        lappend ClustPath(current,elems) "$iElem $jElem"
                        lappend ClustPath(current,seqs) $nextSeqs
                        lappend ClustPath(current,scores) $nextScores
                        set addClust 1
                    }
                    
                } else {
                    set seqs [lindex $ClustPath(current,seqs) 0]
                    set scores [lindex $ClustPath(current,scores) 0]
                    if {$seqs == {}} {
                        set nextSeqs $Clust($iClust,$iElem,$dir,seqs)
                        set nextScores $Clust($iClust,$iElem,$dir,scores)
                    } else {
                        foreach seq $seqs score $scores {
                            set nextSeq [expr $seq-1]
                            set index [lsearch $Clust($iClust,$iElem,$dir,seqs) $nextSeq]
                            if {$index != -1} {
                                lappend nextSeqs $nextSeq
                                lappend nextScores [lindex $Clust($iClust,$iElem,$dir,scores) $index]
                            }
                        }
                    }
                    
                    if {[llength $nextSeqs] > 0} {
                        set ClustPath(current) [linsert $ClustPath(current) 0  "$jClust $iClust"]
                        set ClustPath(current,elems) [linsert $ClustPath(current,elems) 0  "$jElem $iElem"]
                        set ClustPath(current,seqs) [linsert $ClustPath(current,seqs) 0 $nextSeqs]
                        set ClustPath(current,scores) [linsert $ClustPath(current,scores) 0 $nextScores]
                        set addClust 1
                    }
                    
                }
                if {$addClust}  {
                    if {1} {
                        set nFrag $Fragment(nFrag)
                      #  puts "$nFrag $ClustPath(current)"
                        set Fragment($nFrag,clst) $ClustPath(current)
                        set Fragment($nFrag,link) $ClustPath(current,elems)
                        set Fragment($nFrag,seqs) $ClustPath(current,seqs)
                        set Fragment($nFrag,scores) $ClustPath(current,scores)
                    }
                    SetPeakUse $clust $dir $peakStart 1
                    set result [VisitClust $jClust $jElem $dir]
                  #  puts "result $pastResult $result"
                    if {$result==1} {
                        incr nAdded
                    }
                    if {$result != $pastResult} {
                        incr Fragment(nFrag)
                        set gotEnd 1
                    } else {
                        set gotEnd 0
                    }
                    set pastResult $result
                   # puts "post $result $ClustPath(current)"
                } else {
                }
                
            }
        }
        set nextSeqs ""
        #	puts "$addClust $nAdded $Fragment(nFrag) $gotEnd"
        if {($gotEnd == 1) && ($Fragment(nFrag) > 0)} {
            #puts addend
            #parray Fragment
            if {$Clust($iClust,-1,$dir,seqs) != {}} {
             #   puts $Clust($iClust,-1,$dir,seqs)
                
                if {$dir == "suc"} {
                    set pFrag [expr $Fragment(nFrag)-1]
                    set seqs [lindex $Fragment($pFrag,seqs) end]
                    set scores [lindex $ClustPath(current,scores) end]
                  #  puts $seqs
                    if {$seqs == {}} {
                        
                        set nextSeqs $Clust($iClust,-1,$dir,seqs)
                        set nextScores $Clust($iClust,-1,$dir,scores)
                    } else {
                        foreach seq $seqs score $scores {
                            set nextSeq [expr $seq+1]
                            set index [lsearch $Clust($iClust,-1,$dir,seqs) $nextSeq]
                            if {$index != -1} {
                                lappend nextSeqs $nextSeq
                                lappend nextScores [lindex $Clust($iClust,-1,$dir,scores) $index]
                            }
                        }
                    }
                    if {[llength $nextSeqs] > 0} {
                        set nFrag $Fragment(nFrag)
                        set Fragment($nFrag,clst) $Fragment($pFrag,clst)
                        lappend Fragment($nFrag,clst) "$iClust {}"
                        set Fragment($nFrag,link) [concat $Fragment($pFrag,link) [list "{} {}"]]
                        set Fragment($nFrag,seqs) [concat $Fragment($pFrag,seqs) $nextSeqs]
                        set Fragment($nFrag,scores) [concat $Fragment($pFrag,scores) $nextScores]
                        incr Fragment(nFrag)
                    }
                    
                } else {
                    set pFrag [expr $Fragment(nFrag)-1]
                    set seqs [lindex $Fragment($pFrag,seqs) 0]
                    set scores [lindex $ClustPath(current,scores) 0]
                    if {$seqs == {}} {
                        
                        set nextSeqs $Clust($iClust,-1,$dir,seqs)
                        set nextScores $Clust($iClust,-1,$dir,scores)
                    } else {
                        foreach seq $seqs score $scores {
                            set nextSeq [expr $seq-1]
                            set index [lsearch $Clust($iClust,-1,$dir,seqs) $nextSeq]
                            if {$index != -1} {
                                lappend nextSeqs $nextSeq
                                lappend nextScores [lindex $Clust($iClust,-1,$dir,scores) $index]
                            }
                        }
                    }
                    if {[llength $nextSeqs] > 0} {
                        set nFrag $Fragment(nFrag)
                        set Fragment($nFrag,clst) [concat [list "{} $iClust"] $Fragment($pFrag,clst)]
                        set Fragment($nFrag,link) [concat [list "{} {}"] $Fragment($pFrag,link)]
                        set Fragment($nFrag,seqs) [concat [list $nextSeqs] $Fragment($pFrag,seqs)]
                        set Fragment($nFrag,scores) [concat [list $nextScores] $Fragment($pFrag,scores)]
                        incr Fragment(nFrag)
                    }
                    
                }
            }
        }
        
        
        
        if {[llength $ClustPath(current)] == 0} {return 0}
        set kClust [lindex [lindex $ClustPath(current) end] 0]
        foreach clust $Clust($kClust,$dir) {
            set confirmed [lindex $clust 2]
            if {$confirmed <= 0} {continue}
            SetPeakUse $clust $dir $peakStart 0
        }
        
        if {$dir == "suc"} {
            set ClustPath(current) [lrange $ClustPath(current) 0 [expr {[llength $ClustPath(current)]-2}]]
            set ClustPath(current,elems) [lrange $ClustPath(current,elems) 0 [expr {[llength $ClustPath(current,elems)]-2}]]
            set ClustPath(current,seqs) [lrange $ClustPath(current,seqs) 0 [expr {[llength $ClustPath(current,seqs)]-2}]]
            set ClustPath(current,scores) [lrange $ClustPath(current,scores) 0 [expr {[llength $ClustPath(current,scores)]-2}]]
        } else {
            set ClustPath(current) [lrange $ClustPath(current) 1 end]
            set ClustPath(current,elems) [lrange $ClustPath(current,elems) 1 end]
            set ClustPath(current,seqs) [lrange $ClustPath(current,seqs) 1 end]
            set ClustPath(current,scores) [lrange $ClustPath(current,scores) 1 end]
        }
        return $nAdded
    }
    
    proc CheckPeakUse {clust peakStart dir} {
        global PeaksUsed
        set used 0
        set peakLast [expr {[llength $clust]-2}]
        foreach peakPair [lrange $clust $peakStart $peakLast] {
            if {$peakPair == {}} {continue}
            if {$dir == "suc"} {
                set peak [lindex $peakPair 1] ; #sets $peak to the peak in the suc clust that is paired with one in iClust 
            } else {
                set peak [lindex $peakPair 0] ; #sets $peak to the peak in the pre clust that is paired with one in iClust 
            }
            
            if {[info exists PeaksUsed($peak)] && $PeaksUsed($peak) } {
                set used 1
                break
            }
            
            
            if {$used} {break}
        }
        return $used
    }
    proc SetPeakUse {clust dir peakStart state} {
        global PeaksUsed
        set peakLast [expr {[llength $clust]-2}]
        foreach peakPair [lrange $clust $peakStart $peakLast] {
            if {$dir == "suc"} {
                set peak [lindex $peakPair 1]
            } else {
                set peak [lindex $peakPair 0]
            }
            set PeaksUsed($peak) $state
        }
    }
}
